/***************************************************************************
 Copyright (C) 2007 Christoph Reichenbach <reichenb@colorado.edu>

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public Licence as
 published by the Free Software Foundaton; either version 2 of the
 Licence, or (at your option) any later version.

 It is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 merchantibility or fitness for a particular purpose. See the
 GNU General Public Licence for more details.

 You should have received a copy of the GNU General Public Licence
 along with this program; see the file COPYING. If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.

***************************************************************************/

package edu.colorado.cs;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.util.Map;
import java.util.HashMap;

public class AdvParser {
	public AdvParser(InputStream in)
	{
		reader = new BufferedReader(new InputStreamReader(System.in));
	};

	protected BufferedReader reader;

	protected Map<String, Direction> directionMap = new HashMap<String, Direction>();
	protected Direction[] directions = { Direction.NORTH, Direction.SOUTH, Direction.EAST, Direction.WEST };

	{
		for (int i = 0; i < directions.length; i++)
			directionMap.put(directions[i].toString(), directions[i]);
		for (int i = 0; i < directions.length; i++)
			directionMap.put(directions[i].toShortString(), directions[i]);
	}

	public boolean
	parse()
	{
		status();

		System.out.print("> ");
		String input = "";

		try {
			input = reader.readLine().trim().toUpperCase();
		} catch (java.io.IOException _) {
			System.out.println("Aborted...");
			System.exit(0);
		}

		String cmd_string, param_string;
		final int blank_index = input.indexOf(' ');

		if (blank_index == -1) {
			cmd_string = input.intern();
			param_string = null;
		} else {
			cmd_string = input.substring(0, blank_index).trim().intern();
			param_string = input.substring(blank_index).trim().intern();
		}

		if (cmd_string == "QUIT"
		    || cmd_string == "EXIT") {
			quit();
			return false;
		}

		if (directionMap.containsKey(cmd_string)) {
		        param_string = cmd_string;
			cmd_string = "GO";
		}

		if (cmd_string == "GO") {
			Direction dir = directionMap.get(param_string);

			/* May be null! */

			go(dir);
		} else if (cmd_string == "LOOK" || cmd_string == "L")
			look(param_string);
		else if (cmd_string == "GET" || cmd_string == "T" || cmd_string == "TAKE")
			get(param_string);
		else if (cmd_string == "DROP")
			drop(param_string);
		else if (cmd_string == "USE" || cmd_string == "APPLY")
			use(param_string);
		else if (cmd_string == "INVENTORY" || cmd_string == "INV" || cmd_string == "I")
			inventory();
		else System.out.println("Sorry, but I have no idea what ``" + cmd_string + "'' means.");

		return true;
	}

	public void
	status()
	{
		System.out.println("This here should be a status message after you're done with 2.b...");
	}

	public void
	look(String s)
	{
		System.out.println("My eyes are missing");
	}

	public void
	go(Direction d)
	{
		System.out.println("I don't know how to ``go'' anywhere yet");
	}

	public void
	get(String item)
	{
		System.out.println("I have no hands yet...");
	}

	public void
	drop(String item)
	{
		System.out.println("How do I do that?");
	}

	public void
	use(String item)
	{
		System.out.println("I don't have any items.");
	}

	public void
	inventory()
	{
		System.out.println("I don't even have an inventory...");
	}

	public void
	quit()
	{
		System.out.println("Goodbye!");
	}

}
